
#import "Phero2d.h"
#import <math.h>
#include <random.h>

//objekt phero2d realizuje priestor voni

@implementation Phero2d

const unsigned maxStates = 0x7fff;

PHASE(Creating)

+ createBegin: aZone
{
  Phero2d *r = [super createBegin: aZone];

  r->diffusionConstant = 1.0;
  r->evaporationRate = 1.0;
  r->numStates = maxStates;
  r->type = 0;

  return r;
}

//nastavenie pociatocneho mnozstva jedla (kvoli EZGraphu)

-setAmount:(int) i
{
  amount=amountPrev=i;
  return self;
}

//nastavenie odkazu na arenu
//typ udava ci sa jedna o vonu jedla (typ 2)
//alebo vonu feromonu (typ 1)
//z areny sa cerpe info o rozmiesneni prekazok
//pod prekazkou sa vona nesiri
//
//v pripade typu 2 (jedlo) phero2d berie info aj o umiestneni
//jedla v arene
-setArena:(id) f setType:(int)typ
{

  arena=f;
  type=typ;
 
  return self;
}

//nastavenie konstanty difuzie
- setDiffusionConstant: (double)d
{
  diffusionConstant = d;

  return self;
}

//nastavenie vyparovacej konstanty
- setEvaporationRate: (double)e
{
  evaporationRate = e;

  return self;
}


//inicializacia mriezky
- initializeLattice
{
  unsigned x, y;

  for (y = 0; y < ysize; y++)
    for (x = 0; x < xsize; x++)
      {
        long newState;

        newState = 0L;

        [self putValue: newState atX: x Y: y];
      }
  [self updateLattice];

  return self;
}


//stepRule vypocitava novu hodnotu vsetkych buniek
//pre kazdy krok simulacie

- stepRule
{
  unsigned x, y;
  int i,j,pom=0;
  
  for (x = 0; x < xsize-1; x++)
    for (y = 0; y < ysize-1; y++)
      {
        double delta, d;
        long sum;
        long newState;
        unsigned xm1, xp1, ym1, yp1;
        
      if(type==1)  //vona feromonu
      {
        if([arena getValueAtX: x Y:y]==66)  //ak nasiel prakazku, bunka znulovana
          *(discrete2dSiteAt (newLattice, offsets, x, y)) = 0;
        else
        {      
          xm1 = (x - 1);
          if(xm1<=0) xm1 = 0;
          xp1 = (x + 1);
          if(xp1>=xsize-1) xp1 = xsize-1;
          ym1 = (y - 1);
          if(ym1<=0) ym1 = 0;
          yp1 = (y + 1);
          if(yp1>=ysize-1) yp1 = ysize-1;
        
          sum = 0;

          sum += (long) *(discrete2dSiteAt(lattice, offsets, xm1, ym1));     
          sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, x, ym1))); 
          sum += (long) *(discrete2dSiteAt(lattice, offsets, xp1, ym1));     
          sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, xm1, y))); 
          sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, xp1, y))); 
          sum += (long) *(discrete2dSiteAt(lattice, offsets, xm1, yp1));     
          sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, x, yp1))); 
          sum += (long) *(discrete2dSiteAt(lattice, offsets, xp1, yp1));     
          sum -= 20 * (long) (*(discrete2dSiteAt(lattice, offsets, x, y)));
          delta = (double) sum / 20.0;

       
          // prirastok od okolia
          d = ((double)((long)*(discrete2dSiteAt(lattice, offsets, x, y)))) +
            delta * diffusionConstant;
        
          // odparenie
          d *= evaporationRate;

          if (d < 0)
            newState = 0L;
          else if (d + 0.5 >= maxStates)
            newState = maxStates;
          else
            newState = (long)floor (d+0.5);	
          if(x>1 && x<xsize-2 && y>1 && y<ysize-2 )
            *(discrete2dSiteAt (newLattice, offsets, x, y)) = (id)newState;
          else
            *(discrete2dSiteAt (newLattice, offsets, x, y)) = (id)0;
         }        
       }
     else 
      if(type==2)  //typ 2 - vona jedla
      {
            
          xm1 = (x - 1);
          if(xm1<=0) xm1 = 0;
          xp1 = (x + 1);
          if(xp1>=xsize-1) xp1 = xsize-1;
          ym1 = (y - 1);
          if(ym1<=0) ym1 = 0;
          yp1 = (y + 1);
          if(yp1>=ysize-1) yp1 = ysize-1;
        
        sum = 0;

        sum += (long) *(discrete2dSiteAt(lattice, offsets, xm1, ym1));     
        sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, x, ym1))); 
        sum += (long) *(discrete2dSiteAt(lattice, offsets, xp1, ym1));     
        sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, xm1, y))); 
        sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, xp1, y))); 
        sum += (long) *(discrete2dSiteAt(lattice, offsets, xm1, yp1));     
        sum += 4 * (long) (*(discrete2dSiteAt(lattice, offsets, x, yp1)));
        sum += (long) *(discrete2dSiteAt(lattice, offsets, xp1, yp1));    
        sum -= 20 * (long) (*(discrete2dSiteAt(lattice, offsets, x, y)));
        delta = (double) sum / 20.0;
  
      
        // prirastok od okolia 
        d = ((double)((long)*(discrete2dSiteAt(lattice, offsets, x, y)))) +
          delta * diffusionConstant;
        
        // odparenie
        d *= evaporationRate;

        if (d < 0)
          newState = 0L;
        else if (d + 0.5 >= maxStates)
          newState = maxStates;
        else
          newState = (long)floor (d+0.5);

        if( [arena getValueAtX: x Y:y]==64)
        {
          newState = 25000;            //ak nasiel jedlo priadj vonu
          pom++;			//pom pocita mnozstvo nezjedeneho jedla
        }
        if( [arena getValueAtX: x Y:y]==66)
          newState = 0;           //ak nasiel prekazku, znuluj hodnotu  bunky

        if(x>1 && x<xsize-2 && y>1 && y<ysize-2 )
            *(discrete2dSiteAt (newLattice, offsets, x, y)) = (id)newState;
          else
            *(discrete2dSiteAt (newLattice, offsets, x, y)) = (id)0;       }        
      } 

  amountPrev=amount;
  amount=pom;              //mnozstvo jedla
  [self updateLattice];
  return self;
}

-(int)getAmount
{
  
  return amount;        // posle mnozstvo jedla EZgrafu
}

-(int)getAmountDiff
{

  return amountPrev-amount;   // posle mnozstvo jedla EZgrafu
}
@end
